@@ -219,14 +219,14 @@ static void v4l1_find_min_and_max_size(int index, struct v4l2_format *fmt2)
   for (i = 0; ; i++) {
     fmtdesc2.index = i;
 
-    if (SYS_IOCTL(devices[index].fd, VIDIOC_ENUM_FMT, &fmtdesc2))
+    if (v4l2_ioctl(devices[index].fd, VIDIOC_ENUM_FMT, &fmtdesc2))
       break;
 
     fmt2->fmt.pix.pixelformat = fmtdesc2.pixelformat;
     fmt2->fmt.pix.width = 48;
     fmt2->fmt.pix.height = 32;
 
-    if (SYS_IOCTL(devices[index].fd, VIDIOC_TRY_FMT, fmt2) == 0) {
+    if (v4l2_ioctl(devices[index].fd, VIDIOC_TRY_FMT, fmt2) == 0) {
       if (fmt2->fmt.pix.width < devices[index].min_width)
 	devices[index].min_width = fmt2->fmt.pix.width;
       if (fmt2->fmt.pix.height < devices[index].min_height)
@@ -237,7 +237,7 @@ static void v4l1_find_min_and_max_size(int index, struct v4l2_format *fmt2)
     fmt2->fmt.pix.width = 100000;
     fmt2->fmt.pix.height = 100000;
 
-    if (SYS_IOCTL(devices[index].fd, VIDIOC_TRY_FMT, fmt2) == 0) {
+    if (v4l2_ioctl(devices[index].fd, VIDIOC_TRY_FMT, fmt2) == 0) {
       if (fmt2->fmt.pix.width > devices[index].max_width)
 	devices[index].max_width = fmt2->fmt.pix.width;
       if (fmt2->fmt.pix.height > devices[index].max_height)
@@ -307,7 +307,7 @@ int v4l1_open (const char *file, int oflag, ...)
 
   /* Register with libv4l2, as we use that todo format conversion and read()
      emulation for us */
-  if (v4l2_fd_open(fd, V4L2_ENABLE_ENUM_FMT_EMULATION) == -1) {
+  if (v4l2_fd_open(fd, 0) == -1) {
     int saved_err = errno;
     SYS_CLOSE(fd);
     errno = saved_err;
@@ -542,24 +542,27 @@ int v4l1_ioctl (int fd, unsigned long int request, ...)
       break;
 
     case VIDIOCSWIN:
+    case VIDIOCGWIN:
       {
 	struct video_window *win = arg;
 
 	devices[index].flags |= V4L1_PIX_SIZE_TOUCHED;
 
-	result = v4l1_set_format(index, win->width, win->height, -1, 1);
+	if (request == VIDIOCSWIN)
+	  result = v4l1_set_format(index, win->width, win->height, -1, 1);
+	else
+	  result = 0;
+
 	if (result == 0) {
+	  win->x = 0;
+	  win->y = 0;
 	  win->width  = devices[index].width;
 	  win->height = devices[index].height;
+	  win->flags = 0;
 	}
       }
       break;
 
-    case VIDIOCGWIN:
-      devices[index].flags |= V4L1_PIX_SIZE_TOUCHED;
-      result = SYS_IOCTL(fd, request, arg);
-      break;
-
     case VIDIOCGCHAN:
       {
 	struct v4l2_input input2;
