@@ -46,6 +46,7 @@ static const struct v4lconvert_pixfmt supported_src_pixfmts[] = {
   { V4L2_PIX_FMT_YUYV,         0 },
   { V4L2_PIX_FMT_YVYU,         0 },
   { V4L2_PIX_FMT_UYVY,         0 },
+  { V4L2_PIX_FMT_RGB565,       0 },
   { V4L2_PIX_FMT_SN9C20X_I420, V4LCONVERT_NEEDS_CONVERSION },
   { V4L2_PIX_FMT_SBGGR8,       V4LCONVERT_NEEDS_CONVERSION },
   { V4L2_PIX_FMT_SGBRG8,       V4LCONVERT_NEEDS_CONVERSION },
@@ -63,6 +64,8 @@ static const struct v4lconvert_pixfmt supported_src_pixfmts[] = {
   { V4L2_PIX_FMT_MR97310A,     V4LCONVERT_COMPRESSED },
   { V4L2_PIX_FMT_SQ905C,       V4LCONVERT_COMPRESSED },
   { V4L2_PIX_FMT_PJPG,         V4LCONVERT_COMPRESSED },
+  { V4L2_PIX_FMT_OV511,        V4LCONVERT_COMPRESSED },
+  { V4L2_PIX_FMT_OV518,        V4LCONVERT_COMPRESSED },
 };
 
 static const struct v4lconvert_pixfmt supported_dst_pixfmts[] = {
@@ -96,6 +99,7 @@ struct v4lconvert_data *v4lconvert_create(int fd)
     return NULL;
 
   data->fd = fd;
+  data->decompress_pid = -1;
 
   /* Check supported formats */
   for (i = 0; ; i++) {
@@ -158,6 +162,7 @@ void v4lconvert_destroy(struct v4lconvert_data *data)
     tinyjpeg_set_components(data->jdec, comps, 3);
     tinyjpeg_free(data->jdec);
   }
+  v4lconvert_helper_cleanup(data);
   free(data->convert1_buf);
   free(data->convert2_buf);
   free(data->rotate90_buf);
@@ -355,7 +360,7 @@ int v4lconvert_try_format(struct v4lconvert_data *data,
   int i, result;
   unsigned int desired_width = dest_fmt->fmt.pix.width;
   unsigned int desired_height = dest_fmt->fmt.pix.height;
-  struct v4l2_format try_src, try_dest;
+  struct v4l2_format try_src, try_dest, try2_src, try2_dest;
 
   if (dest_fmt->type == V4L2_BUF_TYPE_VIDEO_CAPTURE &&
       v4lconvert_supported_dst_fmt_only(data) &&
@@ -374,6 +379,29 @@ int v4lconvert_try_format(struct v4lconvert_data *data,
     return result;
   }
 
+  /* In case of a non exact resolution match, try again with a slightly larger
+     resolution as some weird devices are not able to crop of the number of
+     extra (border) pixels most sensors have compared to standard resolutions,
+     which we will then just crop of in software */
+  if (try_dest.fmt.pix.width != desired_width ||
+      try_dest.fmt.pix.height != desired_height) {
+    try2_dest = *dest_fmt;
+    try2_dest.fmt.pix.width  = desired_width + 7;
+    try2_dest.fmt.pix.height = desired_height + 1;
+    result = v4lconvert_do_try_format(data, &try2_dest, &try2_src);
+    if (result == 0 &&
+	try2_dest.fmt.pix.width >= desired_width &&
+	try2_dest.fmt.pix.width <= desired_width + 7 &&
+	try2_dest.fmt.pix.height >= desired_height &&
+	try2_dest.fmt.pix.height <= desired_height + 1) {
+      /* Success! */
+      try2_dest.fmt.pix.width = desired_width;
+      try2_dest.fmt.pix.height = desired_height;
+      try_dest = try2_dest;
+      try_src = try2_src;
+    }
+  }
+
   /* In case of a non exact resolution match, see if this is a well known
      resolution some apps are hardcoded too and try to give the app what it
      asked for by cropping a slightly larger resolution or adding a small
@@ -383,7 +411,7 @@ int v4lconvert_try_format(struct v4lconvert_data *data,
     for (i = 0; i < ARRAY_SIZE(v4lconvert_crop_res); i++) {
       if (v4lconvert_crop_res[i][0] == desired_width &&
 	  v4lconvert_crop_res[i][1] == desired_height) {
-	struct v4l2_format try2_src, try2_dest = *dest_fmt;
+	try2_dest = *dest_fmt;
 
 	/* Note these are chosen so that cropping to vga res just works for
 	   vv6410 sensor cams, which have 356x292 and 180x148 */
@@ -419,20 +447,13 @@ int v4lconvert_try_format(struct v4lconvert_data *data,
 
   /* Some applications / libs (*cough* gstreamer *cough*) will not work
      correctly with planar YUV formats when the width is not a multiple of 8
-     or the height is not a multiple of 2 */
-  if (try_dest.fmt.pix.pixelformat == V4L2_PIX_FMT_YUV420 ||
-      try_dest.fmt.pix.pixelformat == V4L2_PIX_FMT_YVU420) {
-    try_dest.fmt.pix.width &= ~7;
-    try_dest.fmt.pix.height &= ~1;
-  }
+     or the height is not a multiple of 2. With RGB formats these apps require
+     the width to be a multiple of 4. We apply the same rounding to all
+     formats to not end up with 2 close but different resolutions. */
+  try_dest.fmt.pix.width &= ~7;
+  try_dest.fmt.pix.height &= ~1;
 
-  /* Likewise the width needs to be a multiple of 4 for RGB formats
-     (although I've never seen a device with a width not a multiple of 4) */
-  if (try_dest.fmt.pix.pixelformat == V4L2_PIX_FMT_RGB24 ||
-      try_dest.fmt.pix.pixelformat == V4L2_PIX_FMT_BGR24)
-    try_dest.fmt.pix.width &= ~3;
-
-  /* Are we converting? */
+  /* Are we converting / cropping ? */
   if(try_src.fmt.pix.width != try_dest.fmt.pix.width ||
      try_src.fmt.pix.height != try_dest.fmt.pix.height ||
      try_src.fmt.pix.pixelformat != try_dest.fmt.pix.pixelformat)
@@ -504,7 +525,7 @@ static unsigned char *v4lconvert_alloc_buffer(struct v4lconvert_data *data,
 }
 
 static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
-  unsigned char *src, int src_size, unsigned char *dest,
+  unsigned char *src, int src_size, unsigned char *dest, int dest_size,
   struct v4l2_format *fmt, unsigned int dest_pix_fmt)
 {
   unsigned int header_width, header_height;
@@ -615,8 +636,11 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
     case V4L2_PIX_FMT_SPCA505:
     case V4L2_PIX_FMT_SPCA508:
     case V4L2_PIX_FMT_SN9C20X_I420:
+    case V4L2_PIX_FMT_OV511:
+    case V4L2_PIX_FMT_OV518:
     {
       unsigned char *d;
+      int d_size;
       int yvu = 0;
 
       if (dest_pix_fmt != V4L2_PIX_FMT_YUV420 &&
@@ -625,8 +649,11 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	      &data->convert_pixfmt_buf, &data->convert_pixfmt_buf_size);
 	if (!d)
 	  return -1;
-      } else
+	d_size = width * height * 3 / 2;
+      } else {
 	d = dest;
+	d_size = dest_size;
+      }
 
       if (dest_pix_fmt == V4L2_PIX_FMT_YVU420)
 	yvu = 1;
@@ -644,6 +671,22 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	case V4L2_PIX_FMT_SN9C20X_I420:
 	  v4lconvert_sn9c20x_to_yuv420(src, d, width, height, yvu);
 	  break;
+	case V4L2_PIX_FMT_OV511:
+	  if (v4lconvert_helper_decompress(data, LIBDIR "/libv4l/ov511-decomp",
+		     src, src_size, d, d_size, width, height, yvu)) {
+	    /* Corrupt frame, better get another one */
+	    errno = EAGAIN;
+	    return -1;
+	  }
+	  break;
+	case V4L2_PIX_FMT_OV518:
+	  if (v4lconvert_helper_decompress(data, LIBDIR "/libv4l/ov518-decomp",
+		     src, src_size, d, d_size, width, height, yvu)) {
+	    /* Corrupt frame, better get another one */
+	    errno = EAGAIN;
+	    return -1;
+	  }
+	  break;
       }
 
       switch (dest_pix_fmt) {
@@ -702,7 +745,12 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	  tmpfmt.fmt.pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
 	  break;
 	case V4L2_PIX_FMT_PAC207:
-	  v4lconvert_decode_pac207(src, tmpbuf, width, height);
+	  if (v4lconvert_decode_pac207(data, src, src_size, tmpbuf,
+				       width, height)) {
+	    /* Corrupt frame, better get another one */
+	    errno = EAGAIN;
+	    return -1;
+	  }
 	  tmpfmt.fmt.pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
 	  break;
 	case V4L2_PIX_FMT_MR97310A:
@@ -745,6 +793,23 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
       }
       break;
 
+    case V4L2_PIX_FMT_RGB565:
+      switch (dest_pix_fmt) {
+      case V4L2_PIX_FMT_RGB24:
+	v4lconvert_rgb565_to_rgb24(src, dest, width, height);
+	break;
+      case V4L2_PIX_FMT_BGR24:
+	v4lconvert_rgb565_to_bgr24(src, dest, width, height);
+	break;
+      case V4L2_PIX_FMT_YUV420:
+	v4lconvert_rgb565_to_yuv420(src, dest, fmt, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_rgb565_to_yuv420(src, dest, fmt, 1);
+	break;
+      }
+      break;
+
     case V4L2_PIX_FMT_RGB24:
       switch (dest_pix_fmt) {
       case V4L2_PIX_FMT_BGR24:
@@ -878,7 +943,9 @@ int v4lconvert_convert(struct v4lconvert_data *data,
   int res, dest_needed, temp_needed, processing, convert = 0;
   int rotate90, vflip, hflip, crop;
   unsigned char *convert1_dest = dest;
+  int convert1_dest_size = dest_size;
   unsigned char *convert2_src = src, *convert2_dest = dest;
+  int convert2_dest_size = dest_size;
   unsigned char *rotate90_src = src, *rotate90_dest = dest;
   unsigned char *flip_src = src, *flip_dest = dest;
   unsigned char *crop_src = src;
@@ -955,6 +1022,8 @@ int v4lconvert_convert(struct v4lconvert_data *data,
     if (!convert1_dest)
       return -1;
 
+    convert1_dest_size =
+      my_src_fmt.fmt.pix.width * my_src_fmt.fmt.pix.height * 3;
     convert2_src = convert1_dest;
   }
 
@@ -964,6 +1033,7 @@ int v4lconvert_convert(struct v4lconvert_data *data,
     if (!convert2_dest)
       return -1;
 
+    convert2_dest_size = temp_needed;
     rotate90_src = flip_src = crop_src = convert2_dest;
   }
 
@@ -988,7 +1058,8 @@ int v4lconvert_convert(struct v4lconvert_data *data,
   /* Done setting sources / dest and allocating intermediate buffers,
      real conversion / processing / ... starts here. */
   if (convert == 2) {
-    res = v4lconvert_convert_pixfmt(data, src, src_size, convert1_dest,
+    res = v4lconvert_convert_pixfmt(data, src, src_size,
+				    convert1_dest, convert1_dest_size,
 				    &my_src_fmt,
 				    V4L2_PIX_FMT_RGB24);
     if (res)
@@ -1002,7 +1073,8 @@ int v4lconvert_convert(struct v4lconvert_data *data,
 
   if (convert) {
     res = v4lconvert_convert_pixfmt(data, convert2_src, src_size,
-				    convert2_dest, &my_src_fmt,
+				    convert2_dest, convert2_dest_size,
+				    &my_src_fmt,
 				    my_dest_fmt.fmt.pix.pixelformat);
     if (res)
       return res;
@@ -1114,12 +1186,8 @@ int v4lconvert_enum_framesizes(struct v4lconvert_data *data,
     case V4L2_FRMSIZE_TYPE_DISCRETE:
       frmsize->discrete = data->framesizes[frmsize->index].discrete;
       /* Apply the same rounding algorithm as v4lconvert_try_format */
-      if (frmsize->pixel_format == V4L2_PIX_FMT_YUV420 ||
-	  frmsize->pixel_format == V4L2_PIX_FMT_YVU420) {
-	frmsize->discrete.width &= ~7;
-	frmsize->discrete.height &= ~1;
-      } else
-	frmsize->discrete.width &= ~3;
+      frmsize->discrete.width &= ~7;
+      frmsize->discrete.height &= ~1;
       break;
     case V4L2_FRMSIZE_TYPE_CONTINUOUS:
     case V4L2_FRMSIZE_TYPE_STEPWISE:
